import React, { useEffect, useState } from 'react';
 import { View, Text, Button, FlatList } from 'react-native';
 import { useNavigation } from '@react-navigation/native';
 import Styles from '../styles/Style';
 import Icon from 'react-native-vector-icons/MaterialIcons';
 import Swipeout from 'react-native-swipeout';
 import AsyncStorage from '@react-native-async-storage/async-storage';
 import Constants from 'expo-constants';
 import {
   getItemsWithQuantities,
   deleteItem,
   increaseQuantity,
   decreaseQuantity
 } from '../database/dbUtils';

 export default function CartScreen() {
   const navigation = useNavigation();
   const [items, setItems] = useState([]);
   const [totalPrice, setTotalPrice] = useState(0);
   const apiUrl = Constants.manifest.extra.apiUrl;
   const [isDarkMode, setIsDarkMode] = useState(false);

   const retrieveThemeMode = async () => {
     try {
       const themeMode = await AsyncStorage.getItem('themeMode');
       if (themeMode !== null) {
         setIsDarkMode(themeMode === 'dark');
       }
     } catch (error) {
       console.log('Error retrieving theme mode:', error);
     }
   };

   const toggleTheme = () => {
     const newThemeMode = !isDarkMode;
     setIsDarkMode(newThemeMode);
     AsyncStorage.setItem('themeMode', newThemeMode ? 'dark' : 'light');
   };

   useEffect(() => {
     retrieveThemeMode();
     updateCart();
   }, []);

  const updateCart = async () => {
    try {
      const itemsData = await getItemsWithQuantities();
      setItems(itemsData);
      calculateTotalPrice(itemsData);
    } catch (error) {
      console.log('Error fetching data:', error);
    }
  };


  const calculateTotalPrice = (items) => {
    const cartItems = items.map((item) => ({
      item_id: item.id,
      quantity: item.quantity,
    }));
    console.log(items);
    const requestOptions = {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify(cartItems),
    };
    fetch(`${apiUrl}/items/calculate-total`, requestOptions)
      .then((response) => {
        if (!response.ok) {
          throw new Error('Response not valid');
        }
        return response.json();
      })
      .then((total) => {
        console.log(total);
        setTotalPrice((total / 100).toFixed(2));
      })
      .catch((error) => {
        console.log('Error calculating total:', error);
      });
  };

  const swipeoutBtns = (item) => [{
    text: 'Delete',
    backgroundColor: 'red',
    onPress: () => {
      if (item && item.id) {
        deleteItem(item.id).then(() => {
          updateCart();
        });
      }
    },
  }];

 const renderItem = ({ item }) => {
   return (
     <Swipeout right={swipeoutBtns(item)} autoClose={true}>
       <View style={Styles.itemRow}>
         <View style={Styles.itemInfo}>
           <Text style={Styles.itemName}>{item.name}</Text>
           <Text style={Styles.itemQuantity}>({item.quantity})</Text>
         </View>
         <Text style={Styles.itemPrice}>{(item.price / 100).toFixed(2)} €</Text>
         <View style={Styles.quantityButtons}>
           <Button title="+" onPress={async () => {
             await increaseQuantity(item.id);
             updateCart();
           }} />
           <Button title="-" onPress={async () => {
             await decreaseQuantity(item.id);
             updateCart();
           }} />
         </View>
       </View>
     </Swipeout>
   );
 };

  return (
    <View style={isDarkMode ? Styles.darkContainer : Styles.lightContainer}>
      <FlatList
        data={items}
        renderItem={renderItem}
        style={Styles.flatList}
        keyExtractor={(item) => item.id.toString()}
      />
      <View style={Styles.totalPriceContainer}>
        <Text style={isDarkMode ? Styles.darkText : Styles.lightText}>Total:</Text>
        <Text style={isDarkMode ? Styles.darkText : Styles.lightText}>{totalPrice} €</Text>
      </View>
      <Button
        style={{margin: 20}}
        color="#E0A228"
        title="Pay"
        onPress={() => navigation.navigate('Checkout', { items: items, totalPrice: totalPrice })}
      />
    </View>
  );
}

export default CartScreen;
