import React, { useState, useEffect } from 'react';
import { Text, View, StyleSheet, TouchableOpacity, TextInput, Button, Alert } from 'react-native';
import { BarCodeScanner } from 'expo-barcode-scanner';
import Constants from 'expo-constants';
import { ImageBackground } from 'react-native';
import Styles from '../styles/Style';
import AsyncStorage from '@react-native-async-storage/async-storage';
import { useNavigation } from '@react-navigation/native';
import Icon from 'react-native-vector-icons/MaterialIcons';
 import {
   updateLocalItemQuantity,
   checkIfLocalItemExists,
   insertItem,
 } from '../database/dbUtils';

export default function App() {
  const [hasPermission, setHasPermission] = useState(false);
  const [scanned, setScanned] = useState(false);
  const [manualEntry, setManualEntry] = useState(false);
  const [barcodeData, setBarcodeData] = useState('');
  const [item, setItem] = useState();
  const navigation = useNavigation();
  const [isDarkMode, setIsDarkMode] = useState(false);

  useEffect(() => {
    const getBarCodeScannerPermissions = async () => {
      const storedThemeMode = await AsyncStorage.getItem('themeMode');
      if (storedThemeMode) {
        setIsDarkMode(storedThemeMode === 'dark');
      }

      const { status } = await BarCodeScanner.requestPermissionsAsync();
      const isPermissionGranted = status === 'granted';
      setHasPermission(isPermissionGranted);

      if (!isPermissionGranted) {
        setManualEntry(true);
      }
    };
    getBarCodeScannerPermissions();
  }, []);

  const toggleTheme = () => {
    const newThemeMode = !isDarkMode;
    setIsDarkMode(newThemeMode);
    AsyncStorage.setItem('themeMode', newThemeMode ? 'dark' : 'light');
  };

  const checkItemExistence = async (itemId) => {
    const apiUrl = Constants.expoConfig.extra.apiUrl;

    try {
        const apiUrl = Constants.expoConfig.extra.apiUrl;
        const response = await fetch(`${apiUrl}/items/${itemId}`, {
          method: 'GET',
          headers: {
            'Content-Type': 'application/json',
          },
        });
        if (response.ok) {
          const item = await response.json();
          const existingItem = await checkIfLocalItemExists(item.id);

          if (existingItem) {

            const newQuantity = existingItem.quantity + 1;
            await updateLocalItemQuantity(item.id, newQuantity);
          } else {
            await insertItem(item.id, item.name, item.price);
          }

          alert('Item added to the cart');
        } else {
          alert('Item does not exist in the database');
        }
      } catch (error) {
        console.log('Error checking item existence:', error);
      }
  };

  const handleBarCodeScanned = ({ type, data }) => {
    setScanned(true);
    setBarcodeData(data);
    checkItemExistence(data);
  };

  const handleManualEntry = () => {
    if (barcodeData) {
      checkItemExistence(barcodeData);
    } else {
      Alert.alert('Item Code is required');
    }
  };

  return (
    <View style={isDarkMode ? Styles.darkContainer : Styles.lightContainer}>
      {manualEntry === false ? (
        <View style={Styles.buttonGroup}>
          <Icon.Button
            name="add"
            backgroundColor="#E0A228"
            color="white"
            onPress={() => setManualEntry(true)}
          >
            Enter manually
          </Icon.Button>
          <Icon.Button
            name="schedule"
            backgroundColor="#E0A228"
            color="white"
            onPress={() => navigation.navigate('PaymentHistoryScreen')}
          >
            Payments History
          </Icon.Button>
          <Icon.Button
            name="shopping-cart"
            backgroundColor="#E0A228"
            color="white"
            onPress={() => navigation.navigate('CartScreen')}
          >
            Cart
          </Icon.Button>
        </View>
      ) : hasPermission === null ? (
        <Text>Requesting camera permission</Text>
      ) : hasPermission === false ? (
        <Text>No access to camera</Text>
      ) : null}
      {manualEntry === true ? (
        <ImageBackground
          source={require('../assets/backCard.jpg')}
          style={{ ...Styles.cardBackground, width: '100%' }}
        >
          <View style={Styles.cardContainer}>
            <View style={Styles.card}>
              <View style={Styles.fieldContainer}>
                <Text>Item Code:</Text>
                <TextInput
                  style={isDarkMode ? Styles.darkText : Styles.lightText}
                  placeholder="Enter Item Code"
                  value={barcodeData}
                  onChangeText={(text) => setBarcodeData(text)}
                />
              </View>
              <Button
                title="Add to Cart"
                onPress={handleManualEntry}
                style={Styles.roundedButton}
                color='#f5b841'
              />
              <Button
                title="Cancel"
                onPress={() => setManualEntry(false)}
                style={Styles.roundedButton}
                color='#E0A228'
              />
            </View>
          </View>
        </ImageBackground>
      ) : (
        <View style={Styles.barcodeScannerContainer}>
          <BarCodeScanner
            onBarCodeScanned={scanned ? undefined : handleBarCodeScanned}
            style={StyleSheet.absoluteFillObject}
          />
          {scanned && (
            <TouchableOpacity
              onPress={() => setScanned(false)}
              style={isDarkMode ? Styles.toggleButton : Styles.inactiveToggleButton}
            >
              <View style={isDarkMode ? Styles.darkText : Styles.lightText}>
                <Text>Tap to Scan Again</Text>
              </View>
            </TouchableOpacity>
          )}
        </View>
      )}
      <View style={Styles.themeToggleButtonContainer}>
        <TouchableOpacity
          style={isDarkMode ? Styles.toggleButton : Styles.inactiveToggleButton}
          onPress={toggleTheme}
        >
          <Icon name="brightness-3" size={30} color="grey" />
        </TouchableOpacity>
        <TouchableOpacity
          style={isDarkMode ? Styles.inactiveToggleButton : Styles.toggleButton}
          onPress={toggleTheme}
        >
          <Icon name="wb-sunny" size={30} color="yellow" />
        </TouchableOpacity>
      </View>
    </View>
  );
}
